@@ -1,4 +1,8 @@
 import hashlib
+from unittest.case import skipIf
+
+from integration.helpers.resource import current_region_does_not_support
+from integration.config.service_names import REST_API
 
 try:
     from pathlib import Path
@@ -10,6 +14,7 @@ from parameterized import parameterized
 from integration.helpers.base_test import BaseTest
 
 
+@skipIf(current_region_does_not_support([REST_API]), "Rest API is not supported in this testing region")
 class TestApiSettings(BaseTest):
     def test_method_settings(self):
         self.create_and_verify_stack("combination/api_with_method_settings")
