@@ -5,6 +5,7 @@ import logging
 from time import sleep, time
 import copy
 
+from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model.exceptions import InvalidResourceException
 from samtranslator.plugins import BasePlugin
 from samtranslator.plugins.exceptions import InvalidPluginException
@@ -12,6 +13,11 @@ from samtranslator.public.sdk.resource import SamResourceType
 from samtranslator.public.sdk.template import SamTemplate
 from samtranslator.intrinsics.resolver import IntrinsicsResolver
 from samtranslator.intrinsics.actions import FindInMapAction
+from samtranslator.region_configuration import RegionConfiguration
+
+LOG = logging.getLogger(__name__)
+
+PLUGIN_METRICS_PREFIX = "Plugin-ServerlessApp"
 
 
 class ServerlessAppPlugin(BasePlugin):
@@ -34,12 +40,12 @@ class ServerlessAppPlugin(BasePlugin):
     # CloudFormation times out on transforms after 2 minutes, so setting this
     # timeout below that to leave some buffer
     TEMPLATE_WAIT_TIMEOUT_SECONDS = 105
-    APPLICATION_ID_KEY = 'ApplicationId'
-    SEMANTIC_VERSION_KEY = 'SemanticVersion'
-    LOCATION_KEY = 'Location'
-    TEMPLATE_URL_KEY = 'TemplateUrl'
+    APPLICATION_ID_KEY = "ApplicationId"
+    SEMANTIC_VERSION_KEY = "SemanticVersion"
+    LOCATION_KEY = "Location"
+    TEMPLATE_URL_KEY = "TemplateUrl"
 
-    def __init__(self, sar_client=None, wait_for_template_active_status=False, validate_only=False, parameters={}):
+    def __init__(self, sar_client=None, wait_for_template_active_status=False, validate_only=False, parameters=None):
         """
         Initialize the plugin.
 
@@ -49,18 +55,22 @@ class ServerlessAppPlugin(BasePlugin):
         :param bool validate_only: Flag to only validate application access (uses get_application API instead)
         """
         super(ServerlessAppPlugin, self).__init__(ServerlessAppPlugin.__name__)
+        if parameters is None:
+            parameters = {}
         self._applications = {}
         self._in_progress_templates = []
         self._sar_client = sar_client
         self._wait_for_template_active_status = wait_for_template_active_status
         self._validate_only = validate_only
         self._parameters = parameters
+        self._total_wait_time = 0
 
         # make sure the flag combination makes sense
         if self._validate_only is True and self._wait_for_template_active_status is True:
             message = "Cannot set both validate_only and wait_for_template_active_status flags to True."
             raise InvalidPluginException(ServerlessAppPlugin.__name__, message)
 
+    @cw_timer(prefix=PLUGIN_METRICS_PREFIX)
     def on_before_transform_template(self, template_dict):
         """
         Hook method that gets called before the SAM template is processed.
@@ -73,23 +83,25 @@ class ServerlessAppPlugin(BasePlugin):
         :return: Nothing
         """
         template = SamTemplate(template_dict)
-        intrinsic_resolvers = self._get_intrinsic_resolvers(template_dict.get('Mappings', {}))
+        intrinsic_resolvers = self._get_intrinsic_resolvers(template_dict.get("Mappings", {}))
 
         service_call = None
         if self._validate_only:
             service_call = self._handle_get_application_request
         else:
             service_call = self._handle_create_cfn_template_request
-        for logical_id, app in template.iterate(SamResourceType.Application.value):
+        for logical_id, app in template.iterate({SamResourceType.Application.value}):
             if not self._can_process_application(app):
                 # Handle these cases in the on_before_transform_resource event
                 continue
 
-            app_id = self._replace_value(app.properties[self.LOCATION_KEY],
-                                         self.APPLICATION_ID_KEY, intrinsic_resolvers)
+            app_id = self._replace_value(
+                app.properties[self.LOCATION_KEY], self.APPLICATION_ID_KEY, intrinsic_resolvers
+            )
 
-            semver = self._replace_value(app.properties[self.LOCATION_KEY],
-                                         self.SEMANTIC_VERSION_KEY, intrinsic_resolvers)
+            semver = self._replace_value(
+                app.properties[self.LOCATION_KEY], self.SEMANTIC_VERSION_KEY, intrinsic_resolvers
+            )
 
             if isinstance(app_id, dict) or isinstance(semver, dict):
                 key = (json.dumps(app_id), json.dumps(semver))
@@ -100,22 +112,48 @@ class ServerlessAppPlugin(BasePlugin):
 
             if key not in self._applications:
                 try:
+                    if not RegionConfiguration.is_service_supported("serverlessrepo"):
+                        raise InvalidResourceException(
+                            logical_id, "Serverless Application Repository is not available in this region."
+                        )
                     # Lazy initialization of the client- create it when it is needed
                     if not self._sar_client:
-                        self._sar_client = boto3.client('serverlessrepo')
-                    service_call(app_id, semver, key, logical_id)
+                        self._sar_client = boto3.client("serverlessrepo")
+                    self._make_service_call_with_retry(service_call, app_id, semver, key, logical_id)
                 except InvalidResourceException as e:
                     # Catch all InvalidResourceExceptions, raise those in the before_resource_transform target.
                     self._applications[key] = e
 
+    def _make_service_call_with_retry(self, service_call, app_id, semver, key, logical_id):
+        call_succeeded = False
+        while self._total_wait_time < self.TEMPLATE_WAIT_TIMEOUT_SECONDS:
+            try:
+                service_call(app_id, semver, key, logical_id)
+            except ClientError as e:
+                error_code = e.response["Error"]["Code"]
+                if error_code == "TooManyRequestsException":
+                    LOG.debug("SAR call timed out for application id {}".format(app_id))
+                    sleep_time = self._get_sleep_time_sec()
+                    sleep(sleep_time)
+                    self._total_wait_time += sleep_time
+                    continue
+                else:
+                    raise e
+            call_succeeded = True
+            break
+        if not call_succeeded:
+            raise InvalidResourceException(logical_id, "Failed to call SAR, timeout limit exceeded.")
+
     def _replace_value(self, input_dict, key, intrinsic_resolvers):
         value = self._resolve_location_value(input_dict.get(key), intrinsic_resolvers)
         input_dict[key] = value
         return value
 
     def _get_intrinsic_resolvers(self, mappings):
-        return [IntrinsicsResolver(self._parameters),
-                IntrinsicsResolver(mappings, {FindInMapAction.intrinsic_name: FindInMapAction()})]
+        return [
+            IntrinsicsResolver(self._parameters),
+            IntrinsicsResolver(mappings, {FindInMapAction.intrinsic_name: FindInMapAction()}),
+        ]
 
     def _resolve_location_value(self, value, intrinsic_resolvers):
         resolved_value = copy.deepcopy(value)
@@ -129,10 +167,14 @@ class ServerlessAppPlugin(BasePlugin):
 
         :param dict app: the application and its properties
         """
-        return (self.LOCATION_KEY in app.properties and
-                isinstance(app.properties[self.LOCATION_KEY], dict) and
-                self.APPLICATION_ID_KEY in app.properties[self.LOCATION_KEY] and
-                self.SEMANTIC_VERSION_KEY in app.properties[self.LOCATION_KEY])
+        return (
+            self.LOCATION_KEY in app.properties
+            and isinstance(app.properties[self.LOCATION_KEY], dict)
+            and self.APPLICATION_ID_KEY in app.properties[self.LOCATION_KEY]
+            and app.properties[self.LOCATION_KEY][self.APPLICATION_ID_KEY] is not None
+            and self.SEMANTIC_VERSION_KEY in app.properties[self.LOCATION_KEY]
+            and app.properties[self.LOCATION_KEY][self.SEMANTIC_VERSION_KEY] is not None
+        )
 
     def _handle_get_application_request(self, app_id, semver, key, logical_id):
         """
@@ -146,17 +188,19 @@ class ServerlessAppPlugin(BasePlugin):
         :param string key: The dictionary key consisting of (ApplicationId, SemanticVersion)
         :param string logical_id: the logical_id of this application resource
         """
-        get_application = (lambda app_id, semver: self._sar_client.get_application(
-                                   ApplicationId=self._sanitize_sar_str_param(app_id),
-                                   SemanticVersion=self._sanitize_sar_str_param(semver)))
+        LOG.info("Getting application {}/{} from serverless application repo...".format(app_id, semver))
+        get_application = lambda app_id, semver: self._sar_client.get_application(
+            ApplicationId=self._sanitize_sar_str_param(app_id), SemanticVersion=self._sanitize_sar_str_param(semver)
+        )
         try:
             self._sar_service_call(get_application, logical_id, app_id, semver)
-            self._applications[key] = {'Available'}
+            self._applications[key] = {"Available"}
+            LOG.info("Finished getting application {}/{}.".format(app_id, semver))
         except EndpointConnectionError as e:
             # No internet connection. Don't break verification, but do show a warning.
             warning_message = "{}. Unable to verify access to {}/{}.".format(e, app_id, semver)
-            logging.warning(warning_message)
-            self._applications[key] = {'Unable to verify'}
+            LOG.warning(warning_message)
+            self._applications[key] = {"Unable to verify"}
 
     def _handle_create_cfn_template_request(self, app_id, semver, key, logical_id):
         """
@@ -167,14 +211,16 @@ class ServerlessAppPlugin(BasePlugin):
         :param string key: The dictionary key consisting of (ApplicationId, SemanticVersion)
         :param string logical_id: the logical_id of this application resource
         """
-        create_cfn_template = (lambda app_id, semver: self._sar_client.create_cloud_formation_template(
-            ApplicationId=self._sanitize_sar_str_param(app_id),
-            SemanticVersion=self._sanitize_sar_str_param(semver)
-        ))
+        LOG.info("Requesting to create CFN template {}/{} in serverless application repo...".format(app_id, semver))
+        create_cfn_template = lambda app_id, semver: self._sar_client.create_cloud_formation_template(
+            ApplicationId=self._sanitize_sar_str_param(app_id), SemanticVersion=self._sanitize_sar_str_param(semver)
+        )
         response = self._sar_service_call(create_cfn_template, logical_id, app_id, semver)
+
+        LOG.info("Requested to create CFN template {}/{} in serverless application repo.".format(app_id, semver))
         self._applications[key] = response[self.TEMPLATE_URL_KEY]
-        if response['Status'] != "ACTIVE":
-            self._in_progress_templates.append((response[self.APPLICATION_ID_KEY], response['TemplateId']))
+        if response["Status"] != "ACTIVE":
+            self._in_progress_templates.append((response[self.APPLICATION_ID_KEY], response["TemplateId"]))
 
     def _sanitize_sar_str_param(self, param):
         """
@@ -191,6 +237,7 @@ class ServerlessAppPlugin(BasePlugin):
             return None
         return str(param)
 
+    @cw_timer(prefix=PLUGIN_METRICS_PREFIX)
     def on_before_transform_resource(self, logical_id, resource_type, resource_properties):
         """
         Hook method that gets called before "each" SAM resource gets processed
@@ -215,8 +262,9 @@ class ServerlessAppPlugin(BasePlugin):
             return
 
         # If it is a dictionary, check for other required parameters
-        self._check_for_dictionary_key(logical_id, resource_properties[self.LOCATION_KEY],
-                                       [self.APPLICATION_ID_KEY, self.SEMANTIC_VERSION_KEY])
+        self._check_for_dictionary_key(
+            logical_id, resource_properties[self.LOCATION_KEY], [self.APPLICATION_ID_KEY, self.SEMANTIC_VERSION_KEY]
+        )
 
         app_id = resource_properties[self.LOCATION_KEY].get(self.APPLICATION_ID_KEY)
 
@@ -224,8 +272,11 @@ class ServerlessAppPlugin(BasePlugin):
             raise InvalidResourceException(logical_id, "Property 'ApplicationId' cannot be blank.")
 
         if isinstance(app_id, dict):
-            raise InvalidResourceException(logical_id, "Property 'ApplicationId' cannot be resolved. Only FindInMap "
-                                                       "and Ref intrinsic functions are supported.")
+            raise InvalidResourceException(
+                logical_id,
+                "Property 'ApplicationId' cannot be resolved. Only FindInMap "
+                "and Ref intrinsic functions are supported.",
+            )
 
         semver = resource_properties[self.LOCATION_KEY].get(self.SEMANTIC_VERSION_KEY)
 
@@ -233,8 +284,11 @@ class ServerlessAppPlugin(BasePlugin):
             raise InvalidResourceException(logical_id, "Property 'SemanticVersion' cannot be blank.")
 
         if isinstance(semver, dict):
-            raise InvalidResourceException(logical_id, "Property 'SemanticVersion' cannot be resolved. Only FindInMap "
-                                                       "and Ref intrinsic functions are supported.")
+            raise InvalidResourceException(
+                logical_id,
+                "Property 'SemanticVersion' cannot be resolved. Only FindInMap "
+                "and Ref intrinsic functions are supported.",
+            )
 
         key = (app_id, semver)
 
@@ -257,9 +311,11 @@ class ServerlessAppPlugin(BasePlugin):
         """
         for key in keys:
             if key not in dictionary:
-                raise InvalidResourceException(logical_id, 'Resource is missing the required [{}] '
-                                                           'property.'.format(key))
+                raise InvalidResourceException(
+                    logical_id, "Resource is missing the required [{}] " "property.".format(key)
+                )
 
+    @cw_timer(prefix=PLUGIN_METRICS_PREFIX)
     def on_after_transform_template(self, template):
         """
         Hook method that gets called after the template is processed
@@ -269,51 +325,76 @@ class ServerlessAppPlugin(BasePlugin):
         :param dict template: Dictionary of the SAM template
         :return: Nothing
         """
-        if self._wait_for_template_active_status and not self._validate_only:
-            start_time = time()
-            while (time() - start_time) < self.TEMPLATE_WAIT_TIMEOUT_SECONDS:
-                temp = self._in_progress_templates
-                self._in_progress_templates = []
+        if not self._wait_for_template_active_status or self._validate_only:
+            return
 
-                # Check each resource to make sure it's active
-                for application_id, template_id in temp:
-                    get_cfn_template = (lambda application_id, template_id:
-                                        self._sar_client.get_cloud_formation_template(
-                                            ApplicationId=self._sanitize_sar_str_param(application_id),
-                                            TemplateId=self._sanitize_sar_str_param(template_id)))
+        while self._total_wait_time < self.TEMPLATE_WAIT_TIMEOUT_SECONDS:
+            # Check each resource to make sure it's active
+            LOG.info("Checking resources in serverless application repo...")
+            idx = 0
+            while idx < len(self._in_progress_templates):
+                application_id, template_id = self._in_progress_templates[idx]
+                get_cfn_template = lambda application_id, template_id: self._sar_client.get_cloud_formation_template(
+                    ApplicationId=self._sanitize_sar_str_param(application_id),
+                    TemplateId=self._sanitize_sar_str_param(template_id),
+                )
+
+                try:
                     response = self._sar_service_call(get_cfn_template, application_id, application_id, template_id)
-                    self._handle_get_cfn_template_response(response, application_id, template_id)
+                except ClientError as e:
+                    error_code = e.response["Error"]["Code"]
+                    if error_code == "TooManyRequestsException":
+                        LOG.debug("SAR call timed out for application id {}".format(application_id))
+                        break  # We were throttled by SAR, break out to a sleep
+                    else:
+                        raise e
 
-                # Don't sleep if there are no more templates with PREPARING status
-                if len(self._in_progress_templates) == 0:
-                    break
+                if self._is_template_active(response, application_id, template_id):
+                    self._in_progress_templates.remove((application_id, template_id))
+                else:
+                    idx += 1  # check next template
 
-                # Sleep a little so we don't spam service calls
-                sleep(self.SLEEP_TIME_SECONDS)
+            LOG.info("Finished checking resources in serverless application repo.")
 
-            # Not all templates reached active status
-            if len(self._in_progress_templates) != 0:
-                application_ids = [items[0] for items in self._in_progress_templates]
-                raise InvalidResourceException(application_ids, "Timed out waiting for nested stack templates "
-                                                                "to reach ACTIVE status.")
+            # Don't sleep if there are no more templates with PREPARING status
+            if len(self._in_progress_templates) == 0:
+                break
 
-    def _handle_get_cfn_template_response(self, response, application_id, template_id):
+            # Sleep a little so we don't spam service calls
+            sleep_time = self._get_sleep_time_sec()
+            sleep(sleep_time)
+            self._total_wait_time += sleep_time
+
+        # Not all templates reached active status
+        if len(self._in_progress_templates) != 0:
+            application_ids = [items[0] for items in self._in_progress_templates]
+            raise InvalidResourceException(
+                application_ids, "Timed out waiting for nested stack templates " "to reach ACTIVE status."
+            )
+
+    def _get_sleep_time_sec(self):
+        return self.SLEEP_TIME_SECONDS
+
+    def _is_template_active(self, response, application_id, template_id):
         """
-        Handles the response from the SAR service call
+        Checks the response from a SAR service call; returns True if the template is active,
+        throws an exception if the request expired and returns False in all other cases.
 
         :param dict response: the response dictionary from the app repo
         :param string application_id: the ApplicationId
         :param string template_id: the unique TemplateId for this application
         """
-        status = response['Status']
-        if status != "ACTIVE":
-            # Other options are PREPARING and EXPIRED.
-            if status == 'EXPIRED':
-                message = ("Template for {} with id {} returned status: {}. Cannot access an expired "
-                           "template.".format(application_id, template_id, status))
-                raise InvalidResourceException(application_id, message)
-            self._in_progress_templates.append((application_id, template_id))
+        status = response["Status"]  # options: PREPARING, EXPIRED or ACTIVE
 
+        if status == "EXPIRED":
+            message = "Template for {} with id {} returned status: {}. Cannot access an expired " "template.".format(
+                application_id, template_id, status
+            )
+            raise InvalidResourceException(application_id, message)
+
+        return status == "ACTIVE"
+
+    @cw_timer(prefix="External", name="SAR")
     def _sar_service_call(self, service_call_lambda, logical_id, *args):
         """
         Handles service calls and exception management for service calls
@@ -325,15 +406,11 @@ class ServerlessAppPlugin(BasePlugin):
         """
         try:
             response = service_call_lambda(*args)
-            logging.info(response)
             return response
         except ClientError as e:
-            error_code = e.response['Error']['Code']
-            if error_code in ('AccessDeniedException', 'NotFoundException'):
-                raise InvalidResourceException(logical_id, e.response['Error']['Message'])
-
-            # 'ForbiddenException'- SAR rejects connection
-            logging.exception(e)
+            error_code = e.response["Error"]["Code"]
+            if error_code in ("AccessDeniedException", "NotFoundException"):
+                raise InvalidResourceException(logical_id, e.response["Error"]["Message"])
             raise e
 
     def _resource_is_supported(self, resource_type):
